#!/bin/sh
# 
#############################################################################
##
##    Copyright 2000 Compaq Computer Corporation
##
##    COMPAQ Registered in U.S. Patent and Trademark Office.
##
##    Confidential computer software. Valid license from Compaq or authorized
##    sublicensor required for possession, use or copying. Consistent with
##    FAR 12.211 and 12.212, Commercial Computer Software, Computer Software
##    Documentation, and Technical Data for Commercial Items are licensed to
##    the U.S. Government under vendor's standard commercial license.
##
#############################################################################

#
Make_compconfig_file()
{
    # RELOCATE_LD only if not relying on the ld symbolic link in the
    # bin directory.  This would need to precede the '|' in the
    # comp.config file so the user could override it.
    RELOCATE_LD=""

    # For each include file directory or -L directory searched by
    # gcc by default, change the names of those directories
    # into appropriate -SysIncDir or -L command line arguments.

    # -SysIncDir options for each include directory
    H_DIRS=`echo ' ' $USR_DIST_INC | sed -n -e 's| [^ ][^ ]*| -SysIncDir&|gp'`
    SYS_INCDIR="-SysIncDir ${CCC_ROOT}/include $H_DIRS"

    # For each -SysIncDir directory, generate a -SD option for that directory.
    # Unlike -SysIncDir, which has a whitespace-separated argument, the
    # argument to -SD must be attached.

    SD_OPTIONS=`echo ${SYS_INCDIR} | sed -e 's/-SysIncDir[ 	]*/-SD/g'`

    # -L options for each -L directory used by gcc
    L_DIRS=`echo ' ' $USR_DIST_LIB | sed -n -e 's| \([^ ][^ ]*\)| -L\1|gp'`
    # 
    SYS_LIBDIR="-L${CCC_ROOT}/lib $L_DIRS"

    COMPCONFIG="${RELOCATE_LD} ${SD_OPTIONS} | ${SYS_INCDIR} ${SYS_LIBDIR}"

    # It's not useful to compare the comp.config we're going to generate
    # with one shipped on the kit, because at best they would match for
    # only one distribution.  Just silently generate what we think is
    # right for the system we're on.
    echo "${COMPCONFIG}" > ${CCC_ROOT}/bin/comp.config
    rm -f ${CCC_ROOT}/bin/ld
    ln -s $LDCMD ${CCC_ROOT}/bin/ld

    # Static kit assumes all of the .o's are in /usr/lib.  If a .o
    # of the same name is in the gcc directory we selected, use that
    # one, instead.

    for obj in gcrt1.o crt1.o crti.o crtbegin.o crtend.o crtn.o ; do
	if [ -L ${CCC_ROOT}/bin/${obj} -a -r $1/${obj} ]
	then
	    rm -f ${CCC_ROOT}/bin/${obj}
	    ln -s $1/${obj} ${CCC_ROOT}/bin/${obj}
	fi
    done
}

# ------
usage()
{
    echo "Usage: create-comp-config.sh ccc-version [gcc-path]" >&2
}

cleanup()
{
    rm -f /tmp/ccc_install$$ /tmp/ccc_install$$.*
}

aborting()
{
    cleanup
    cat 1>&2 <<- EOF
	Aborting
	
	If you can correct the problem, you can rerun this script manually
	by entering a command in the following format:
	
	      create-comp-config.sh ccc-version [gcc-path]
	
	The current invocation of create-comp-config.sh was:
	
	$SAVE_PROG_NAME $SAVE_ARGS
	EOF
    exit 1
}


generic_error()
{
    test_failed=$1
    test_file=$2
    error_log=$3

    cat 1>&2 <<- EOF

	The Compaq C Compiler failed its installation verification
	procedure. Please submit a problem report to The University of New
	Hampshire (UNH). UNH, in partnership with Compaq, is developing an
	innovative support network for Alpha Linux developers. UNH supports a
	web site and forums for Alpha Linux developers to exchange information
	and ask questions.  To report problems experienced with Compaq C for 
	Linux Alpha and its components, use the UNH website:
	
	       http://www.iol.unh.edu/linux/alpha/development
	
	UNH has a direct connection to Compaq's compiler and tools development
	organization for fast resolution of identified technical problems.
	You may not receive a personal reply to your query, but we appreciate
	your input on the product.
	
	When submitting your problem, please include the following:
	
	      o All output generated by this installation procedure
	      o The name and version of your Linux distribution
	      o The name of the compiler RPM kit
	      o The command you used to install the kit
	      o The version of GCC installed on your system
	
	EOF

    ccc -V -v 1>&2
    echo " " 1>&2
    
    echo "$test_failed" 1>&2
    if [ ! "$test_file" = "" ]; then
      cat $test_file 1>&2
    fi
    echo " " 1>&2
    cat $error_log 1>&2
    
    aborting
}

run_ccc()
{
  cleanup

  CCC_LOG="/tmp/ccc_install$$.log"
  CCC_EXE="/tmp/ccc_install$$"
  CCC_OBJ="/tmp/ccc_install$$.o"
  CCC_SRC="/tmp/ccc_install$$.c"
  cat > $CCC_SRC <<- EOF
	#include <stddef.h>
	#include <stdio.h>
	int main(void)
	{
	    printf("Hello world.\n");
	    return 0;
	}
	EOF

  sh -cx "/usr/bin/ccc -c $CCC_SRC -o $CCC_OBJ" >  $CCC_LOG 2>&1 || {
      generic_error "Test program compilation:" "$CCC_SRC" "$CCC_LOG"
  }

  sh -cx "/usr/bin/ccc $CCC_OBJ -o $CCC_EXE" >> $CCC_LOG 2>&1 || {
      generic_error "Test program linking:" "$CCC_SRC" "$CCC_LOG"
  }

  sh -cx "$CCC_EXE" >> $CCC_LOG 2>&1 || {
      generic_error "Test program gave non-zero status" "$CCC_SRC" "$CCC_LOG"
  }

  cleanup
}

check_cpml()
{
#
# Verify the version of cpml we have is high enough
#
  minver=$1
  package_high_enough cpml /usr/lib/libcpml.so $minver \
			    > /tmp/install$$.log 2>&1
  status=$?

  if [ $status -eq 2 ]; then 
    cat 1>&2 <<- EOF 
	The Compaq Portable Math Library (CPML) is not installed. See the
	README documentation provided with the compiler kit for instructions
	to obtain a version appropriate for your system, or visit
	http://www.compaq.com/math for the latest version available
	for download.
	EOF
    aborting
  elif [ $status -eq 3 ]; then
    cat 1>&2 <<-EOF
	The Compaq Portable Math Library (CPML) installed on this system is
	not new enough to support this version of the compiler, $minver is
	the minimum cpml version required. See the README documentation
	provided with the compiler kit for instructions to obtain a version
	appropriate for your system, or visit http://www.compaq.com/math
	for the latest version available for download.
	EOF
    aborting 
  elif [ $status -ne 0 ]; then 
    generic_error "CPML Version Identification" "" /tmp/install$$.log
  fi

  cleanup
}

#
# Main Body

SAVE_PROG_NAME="$0"
SAVE_ARGS="$*"

CCC_VERSION="$1"
if [ "$1" = "" ] ; then
    usage
    exit 1
fi

GCC_PATH="$2"

#
# Load the functions used for probing the system information. Do this
# after we grab the arguments for this script, because they will be clobbered.
#
. `dirname $SAVE_PROG_NAME`/probe_linux.sh init

PREFIX=`echo $CCC_VERSION | cut -b 1-3`
if [ "$PREFIX" != "cxx" -a "$PREFIX" != "ccc" ]; then
    usage
    exit 1
fi

CCC_ROOT=${CPQ_COMP_ROOT}/usr/lib/compaq/${CCC_VERSION}/alpha-linux

# Put ccc rpm kit version in bin/.version file for use by ccc -V;
  echo "Installed as ${CCC_VERSION}" > ${CCC_ROOT}/bin/.version

# Put linux distribution identification info in bin/.version file for use 
# by ccc -V;

if [ "$DIST_VENDOR" = "UNKNOWN" ]; then
    echo "This distribution has not been tested at Compaq:" \
	| tee -a ${CCC_ROOT}/bin/.version 1>&2
    if [ ! $DIST_FILES = "" ]; then
        cat $DIST_FILES | tee -a  ${CCC_ROOT}/bin/.version 1>&2
    fi
else
    cat $DIST_FILES >> ${CCC_ROOT}/bin/.version
fi

#
# Find version of GCC to use.
#
if [ "$GCC_PATH" = "" ]; then
  probe_gcc
  if [ ! -d "$GCC_PATH" ]; then
    echo "Error: Unable to find GCC. GCC must be installed." 1>&2
    echo "If GCC is installed, try adding it to your path." 1>&2
    aborting
  fi
fi

if [ ! -r "$GCC_PATH/specs" ]; then
  echo "Error: Unable to find GCC at $GCC_PATH. GCC must be installed." 1>&2
  aborting
fi

#
# What header and lib dirs should we be using? The ones that the specified
# version of gcc is using ...
#
PGM="int main(int argc, char **argv) {return 0;}"
echo "$PGM" > /tmp/ccc_install$$.c
GCC_VER=`basename $GCC_PATH`
env -i PATH=$PATH gcc -v -V $GCC_VER /tmp/ccc_install$$.c \
           -o /tmp/ccc_install$$ > /tmp/ccc_install$$.log 2>&1 || {
    cat - /tmp/ccc_install$$.log <<- EOF 1>&2
	Error: gcc -V $GCC_VER failed to compile and link "$PGM".
	       The ccc installation expects a working installation of GCC.
	EOF
    aborting
}
#
# Get the list of include directories from the list displayed by gcc -v
#
USR_DIST_INC=`sed -n -e '1,/^\#include <...>/d' -e '/^End of search list/q' -e '/^/p' \
                  /tmp/ccc_install$$.log`

# Get the library directories by examining the way gcc invoked ld (or collect2).
# Although we normally see the directories attached to -L, they are allowed
# to be separated by whitespace, so handle both just in case...
LDLINE=`grep -E '/(ld|collect2).*[ 	]-L' /tmp/ccc_install$$.log`
LDCMD=`echo $LDLINE | sed -e 's/^[ 	]*\([^ 	]*\)[ 	]*.*$/\1/'`
if [ ! -x $LDCMD ]; then
    echo "Error: $LDCMD is not executable." 1>&2
    aborting
fi

LSEEN=0
for ARG in $LDLINE ; do
    if [ "$LSEEN" != "0" ]; then
	USR_DIST_LIB="$USR_DIST_LIB $ARG"
	LSEEN=0
    elif [ "x$ARG" = "x-L" ]; then
	LSEEN=1
    elif [ "x`echo $ARG | cut -b 1-2`" = "x-L" ]; then
	USR_DIST_LIB="$USR_DIST_LIB `echo $ARG | cut -b 3-`"
	LSEEN=0
    fi
done
cleanup

# Tested is the list of exact paths we've done the most testing with.
Tested1=/usr/lib/gcc-lib/alpha-redhat-linux/egcs-2.90.29
Tested2=/usr/lib/gcc-lib/alpha-redhat-linux/egcs-2.91.66
Tested3=/usr/lib/gcc-lib/alpha-unknown-linux/egcs-2.91.66
Tested="$Tested1 $Tested2 $Tested3"

# Experimental is a list of pathnames we've recently begun testing with.
Experimental1=/usr/lib/gcc-lib/alpha-suse-linux/2.95.2
Experimental2=/usr/lib/gcc-lib/alpha-redhat-linux/2.96
Experimental3=/usr/lib/gcc-lib/alpha-turbo-linux/2.96
Experimental="$Experimental1 Experimental2 Experimental3"

# UnknownDistTested is a list of GCC versions that were tested ignoring the
# leading pathnames which tend to differ across distributions.
for dir in $Tested ; do
    UnknownDistTested="$UnknownDistTested `basename $dir`"
done

# UnknownDistExperimental is a list of GCC versions that we've done
# some testing with, ignoring the leading pathnames which tend to
# differ across distributions.
for dir in $Experimental ; do
    UnknownDistExperimental="$UnknownDistExperimental `basename $dir`"
done

# Decide the status of the chosen version of gcc, and report it on
# stderr and record it in .version if not specifically something we tested with.
# Note the leading and trailing blanks in strings are required to avoid having
# grep report success for a partial match on a pathname, e.g.
# echo "/usr/lib /usr/foo" | grep "/usr" would match, but
# echo " /usr/lib /usr/foo " | grep " /usr " would not.

if echo " $Tested " | grep -q " ${GCC_PATH} " ; then
    cat - <<- EOF | tee -a ${CCC_ROOT}/bin/.version 1>&2
	Using ${GCC_PATH}.
	EOF
elif echo " $UnknownDistTested " | grep -q " ${GCC_VER} " ; then
    # GCC version tested, different path.
    cat - <<- EOF | tee -a ${CCC_ROOT}/bin/.version 1>&2
	Using ${GCC_PATH} (1).
	EOF
elif echo " ${Experimental} " | grep -q " ${GCC_PATH} " ; then
    # Recently begun testing
    cat - <<- EOF | tee -a ${CCC_ROOT}/bin/.version 1>&2
	Using ${GCC_PATH} (2).
	EOF
elif echo " ${UnknownDistExperimental} " | grep -q " ${GCC_VER} " ; then
    # GCC version recently begun testing, different path.
    cat - <<- EOF | tee -a ${CCC_ROOT}/bin/.version 1>&2
	Using ${GCC_PATH} (3).
	EOF
else
    # Neither GCC version nor path look familiar.
    cat - <<- EOF | tee -a ${CCC_ROOT}/bin/.version 1>&2
	Using ${GCC_PATH} (4).
	EOF
fi

Make_compconfig_file "$GCC_PATH"
run_ccc
check_cpml 5.1.0

exit 0
